once_list2

Struct OnceList

pub struct OnceList<T, A: Allocator = Global> { /* private fields */ }
A single linked list which behaves like std::cell::OnceCell, but for multiple values. See the crate document for the examples.

Implementations

Source

impl<T> OnceList<T, Global>

pub fn new() -> Self

Creates a new empty OnceList. This method does not allocate.

Source

impl<T, A: Allocator> OnceList<T, A>

pub fn new_in(alloc: A) -> Self

Creates a new empty OnceList with the given allocator. This method does not allocate.

pub fn len(&self) -> usize

Returns the number of values in the list. This method is O(n).

pub fn is_empty(&self) -> bool

Returns true if the list is empty.

pub fn contains(&self, val: &T) -> bool
where T: PartialEq,

Returns true if the list contains the value.

pub fn first(&self) -> Option<&T>

Returns a first value, if it exists.

pub fn first_mut(&mut self) -> Option<&mut T>

Returns a mutable reference to the first value, if it exists.

pub fn last(&self) -> Option<&T>

Returns a last value, if it exists. This method is O(n).

pub fn last_mut(&mut self) -> Option<&mut T>

Returns a mutable reference to the last value, if it exists. This method is O(n).

pub fn iter(&self) -> impl Iterator<Item = &T>

Returns an iterator over the &T references in the list.

pub fn iter_mut(&mut self) -> impl Iterator<Item = &mut T>

Returns an iterator over the &mut T references in the list.

pub fn clear(&mut self)

Clears the list, dropping all values.

pub fn allocator(&self) -> &A

Returns an allocator of this struct.

pub fn remove<P>(&mut self, pred: P) -> Option<T>
where P: FnMut(&T) -> bool,

Find a first value in the list matches the predicate, remove that item from the list, and then returns that value.

Source

impl<T: Sized, A: Allocator + Clone> OnceList<T, A>

pub fn push(&self, val: T) -> &T

Appends a value to the list, and returns the reference to that value.

Note that this method takes &self, not &mut self.

pub fn extend<U: IntoIterator<Item = T>>(&self, iter: U)

An almost same method with the std::iter::Extend::extend, though this method takes &self instead of &mut self.